/*
# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image 
#                        pour la Santé)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
#
#  This software is governed by the CeCILL-B license under French law and 
#  abiding by the rules of distribution of free software. You can  use, 
#  modify and/ or redistribute the software under the terms of the CeCILL-B 
#  license as circulated by CEA, CNRS and INRIA at the following URL 
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability. 
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------  
*/

/*=========================================================================
Program:   bbtk
Module:    $RCSfile$
Language:  C++
Date:      $Date$
Version:   $Revision$
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and
*  abiding by the rules of distribution of free software. You can  use,
*  modify and/ or redistribute the software under the terms of the CeCILL-B
*  license as circulated by CEA, CNRS and INRIA at the following URL
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability.
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */

/**
*  \file
*  \brief Class bbtk::wxBlackBoxEditionDialog .
*/

#include "wxBlackBoxEditionDialog.h"
#include "creaWx.h"

namespace bbtk
{
	//=========================================================================

	wxBlackBoxEditionDialog::wxBlackBoxEditionDialog(wxGUIEditorGraphicBBS *parent,GBlackBoxModel *model)
//		:wxDialog(parent,wxID_ANY,_T(""), wxDefaultPosition, wxSize(1200, 1200),wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER) 
		:wxDialog(NULL,wxID_ANY,_T(""), wxDefaultPosition, wxSize(1200, 1200),wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER) 
	{
		_model=model;
		_parent = parent;
		std::string title("BlackBox Editing - ");
		title+=_model->getBBTKPackage();
		title+=":";
		title+=_model->getBBTKType();
		title+=":";
		title+=_model->getBBTKName();
		SetTitle(std2wx(title));

		constructBlackBoxEditionDialog();
	}

	//=========================================================================


	wxBlackBoxEditionDialog::~wxBlackBoxEditionDialog()
	{

	}

	//=========================================================================

	bool wxBlackBoxEditionDialog::isValidNameForABox(std::string boxname) {

         	int i=0;
         	for (i = 0; i < boxname.size() ; i++) {
                 	if ( (isalnum(boxname[i])==0) && (boxname.compare(i, 1, "-") != 0) && ( boxname.compare(i, 1, "_") != 0) ){
                         	return false;
                 	}
         	} // for
 
         	return true;
 	}

	//=========================================================================

	void wxBlackBoxEditionDialog::constructBlackBoxEditionDialog()
	{
		wxBoxSizer *sizerDialog = new wxBoxSizer(wxVERTICAL);

        wxScrolledWindow *scrollWin = new wxScrolledWindow( this, -1, wxDefaultPosition,  wxSize(200,200), wxVSCROLL);

		wxString boxtype( crea::std2wx( _model->getBBTKType() ) );
		
		wxStaticText *textBoxName = new wxStaticText(scrollWin, -1, wxT("Box Name"));
		wxStaticText *textBoxType = new wxStaticText(scrollWin, -1, boxtype+wxT(": ") );
 		wxTextCtrl *valueBoxName  = new wxTextCtrl(scrollWin, -1, _T(""), wxDefaultPosition,wxSize(300,25));
 		_initBoxName = _model->getBBTKName();
 		valueBoxName->SetValue(crea::std2wx(_initBoxName));
 		_boxName = valueBoxName ;

		wxBoxSizer *boxnameSizer        = new wxBoxSizer(wxHORIZONTAL);
		boxnameSizer->Add(textBoxType,0,wxEXPAND,5);
		boxnameSizer->Add(valueBoxName,0,wxEXPAND,5);
		boxnameSizer->Add( new wxStaticText(scrollWin, -1, wxT(" ")) ,0,wxCENTRE|wxEXPAND,5);

		
		
		wxStaticText *text = new wxStaticText(scrollWin, -1, wxT("Input Ports"));
		wxFont font(11, wxDEFAULT, wxNORMAL, wxBOLD);
		text->SetFont(font);
 		textBoxName->SetFont(font);


		std::vector<GPortModel*> lstInputs = _model->getInputPorts();
		wxFlexGridSizer *sizer = new wxFlexGridSizer(lstInputs.size(),3,5,5);
		sizer->AddGrowableCol(0);
		sizer->AddGrowableCol(1);
		sizer->AddGrowableCol(2);
		
		for(int i = 0;i<(int)lstInputs.size();i++)
		{
			sizer->AddGrowableRow(i);
			GPortModel		*port       = lstInputs[i];
			std::string		 type		= port->getBBTKType();
			wxStaticText	*lblName	= new wxStaticText(scrollWin, -1, std2wx(port->getBBTKName()),wxDefaultPosition,wxDefaultSize);
			wxStaticText	*lblType	= new wxStaticText(scrollWin, -1, std2wx(type),wxDefaultPosition,wxSize(250,25));
			wxTextCtrl		*txtValue   = new wxTextCtrl(scrollWin, -1, _T(""),wxDefaultPosition,wxDefaultSize);



			lblName->SetToolTip(  std2wx(port->getBBTKDescription()) );
			txtValue->SetToolTip( std2wx(port->getBBTKDescription()) );
			
			
			if(port->getValue()!="")
			{
				std::string text = port->getValue();
				if(text.length()>0)
					addDoubleQuotes(text);
				txtValue->SetValue(crea::std2wx(text));
			}

			if(port->isConnected())
			{
				std::string connected("--Port Connected--");
				txtValue->SetValue(crea::std2wx(connected));
				txtValue->SetEditable(false);
			}

			char et = '*';
			if(type.find(et)!=-1)
			{
				std::string noEditable("--No editable--");
				txtValue->SetValue(crea::std2wx(noEditable));
				txtValue->SetEditable(false);
			}

			_lstNames.push_back(lblName);
			_lstValues.push_back(txtValue);
			_lstTypes.push_back(lblType);

			sizer->Add(lblName,1,wxEXPAND,5);
			sizer->Add(txtValue,1,wxEXPAND,5);
			sizer->Add(lblType,1,wxCENTRE|wxEXPAND,5);
		}


		wxBoxSizer *buts        = new wxBoxSizer(wxHORIZONTAL);
		wxButton *okButton      = new wxButton(scrollWin, -1, _T("Ok"),wxDefaultPosition, wxSize(70, 30));
		wxButton *closeButton   = new wxButton(scrollWin, -1, _T("Close"), wxDefaultPosition, wxSize(70, 30));

		// connect command event handlers
		Connect(okButton->GetId(),wxEVT_COMMAND_BUTTON_CLICKED,wxCommandEventHandler(wxBlackBoxEditionDialog::onClickOk));
		Connect(closeButton->GetId(),wxEVT_COMMAND_BUTTON_CLICKED,wxCommandEventHandler(wxBlackBoxEditionDialog::onClickClose));

		buts->Add(okButton,0,wxCENTRE|wxEXPAND,5);
		buts->Add(closeButton,0,wxCENTRE|wxEXPAND,5);

		sizerDialog->AddSpacer(10);
		sizerDialog->Add(textBoxName,0,wxALIGN_TOP|wxALIGN_CENTER);
 		sizerDialog->AddSpacer(10);
 		sizerDialog->Add(boxnameSizer, wxSizerFlags(0).Align(0).Border(wxLEFT, 100));
 		sizerDialog->AddSpacer(10);
		sizerDialog->Add(text,0,wxALIGN_TOP|wxALIGN_CENTER);
		sizerDialog->AddSpacer(15);
		sizerDialog->Add(sizer,0,wxALIGN_CENTER| wxEXPAND);
		sizerDialog->AddSpacer(15);
		sizerDialog->Add(buts,0,wxALIGN_CENTER | wxTOP | wxBOTTOM);

		scrollWin->SetSizer(sizerDialog);
		scrollWin->Centre();


        scrollWin->SetVirtualSize(400,400);
        scrollWin->SetSize(300,300);
        scrollWin->SetScrollbars(10, 10, 50, 50);
//        scrollWin->SetSizer(sizer);


		ShowModal();
		Destroy();
	}

	//=========================================================================

	void wxBlackBoxEditionDialog::onClickOk(wxCommandEvent& event)
	{
		int closeok = 1;
		for(int i=0;i<(int)_lstValues.size();i++)
		{
			std::string text = wx2std(_lstValues[i]->GetValue());
			//TOFIX Search a better alternative
			if(text!="--No editable--" && text!="--Port Connected--")
			{
				//JPRG: When executing, add double quotes if necessary
				if(text.length()>0)
				{
					addDoubleQuotes(text);
				}
				_model->setValueToInputPort(i,text);
			}
		}
		//handle box name
 		std::string boxname = wx2std(_boxName->GetValue());
 		if( boxname.compare(_initBoxName) != 0 ){
 			if (isValidNameForABox(boxname) == true){
 				if( _parent->boxNameExists(boxname) == true ){
 					closeok=0;
 					wxMessageDialog *dial = new wxMessageDialog(NULL,
                         wxT("The name already exists. Please provide another name"),
                         wxT("Change name: name already exists"), wxOK | wxICON_EXCLAMATION | wxSTAY_ON_TOP);
 					dial->ShowModal();
 				}else{
 					_model->setBBTKName(boxname);
 				}
 			}else{
 				closeok=0;
 				wxMessageDialog *dial = new wxMessageDialog(NULL,  wxT("Please provide a valide name for your box (no spaces and only leters, digits, \"_\" or \"-\" allowed)"),  wxT("Change name: invalid name"), wxOK | wxICON_EXCLAMATION | wxSTAY_ON_TOP);
 				dial->ShowModal();
 			}
 		}
 
 		if( closeok==1 ){
			_parent->SaveTempActualDiagram("edit values");
			Close(true);
		}
	}

	//=========================================================================

	void wxBlackBoxEditionDialog::onClickClose(wxCommandEvent& event)
	{
		Close(true);
	}

	//=========================================================================

	//=========================================================================
	
	void wxBlackBoxEditionDialog::addDoubleQuotes(std::string &text)
	{
		
		//We add the double quotes at the beginning
		if(text[0] != '"' )
			text = "\"" + text;
		
		//We add the double quotes at the end
		if(text[text.length()-1] != '"')	
			text+= "\"";
		
	}
	

	//=========================================================================
	void wxBlackBoxEditionDialog::removeDoubleQuotes(std::string &text)
	{
		if(text[0]== '"' )
			text.replace(0, 1,"");
		if(text[text.length()-1] == '"')	
			text.replace(text.length()-1, 1,"");
		
	}
	
	
	
}  // EO namespace bbtk

// EOF
/*	// remove * from the name
		std::string name = _tabsMgr->GetNameTabPanel();
		if(name[name.length() - 1] != '*')
			name = name.substr(0, name.length()-2);
		_tabsMgr->SetNameTabPanel(name);*/
